--_________________________________________________________________________________________________
--                                                                                                |
--                                         |The nanoFIP|                                          |
--                                                                                                |
--                                         CERN,BE/CO-HT                                          |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                         dualram_512x8                                          |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         dualram_512x8.vhd                                                                 |
--                                                                                                |
-- Description  A simulation model of the nanoFIP dualram_512x8 module without dependency on the  |
--              PROASIC3 library.                                                                 |
--                o word width: 8 bits and                                                        |
--                o depth     : 512 bytes.                                                        |
--                                                                                                |
-- Authors      Tomasz Wlostowski (Tomasz.Wlostowski@cern.ch)                                     |
--                                                                                                |
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, download it from http://www.gnu.org/licenses/lgpl-2.1.html                     |
---------------------------------------------------------------------------------------------------



--=================================================================================================
--                                      Libraries & Packages
--=================================================================================================

-- Standard library
library IEEE;
use IEEE.STD_LOGIC_1164.all;            -- std_logic definitions
use IEEE.NUMERIC_STD.all;               -- conversion functions
-- Component specific library


--=================================================================================================
--                             Entity declaration for dualram_512x8
--=================================================================================================

entity dualram_512x8 is port(
  -- INPUTS
  -- Inputs concerning port A
  CLKA  : in std_logic;  -- clock A for synchronous read/ write operations
  ADDRA : in std_logic_vector (8 downto 0);  -- address A
  DINA  : in std_logic_vector (7 downto 0);  -- data in A
  RWA   : in std_logic;  -- read/ write mode; 1 for reading, 0 for writing

  -- Inputs concerning port B
  CLKB  : in std_logic;  -- clock B for synchronous read/ write operations
  ADDRB : in std_logic_vector (8 downto 0);  -- address B
  DINB  : in std_logic_vector (7 downto 0);  -- data in B
  RWB   : in std_logic;  -- read/ write mode; 1 for reading, 0 for writing

  -- Reset
  RESETn : in std_logic;  -- sets all outputs low; does not reset the memory


  -- OUTPUTS
  -- Output concerning port A
  DOUTA : out std_logic_vector (7 downto 0);  -- data out A

  -- Output concerning port B
  DOUTB : out std_logic_vector (7 downto 0));  -- data out B
end dualram_512x8;


--=================================================================================================
--                                    architecture declaration
--=================================================================================================
architecture RAM4K9 of dualram_512x8 is

  type t_ram_array is array(0 to 511) of std_logic_vector(7 downto 0);
  shared variable ram : t_ram_array;

begin

  process(CLKA)
  begin
    if rising_edge(CLKA) then
      if(RWA = '0') then
        ram(to_integer(unsigned(ADDRA) mod 512)) := DINA;
      else
        DOUTA <= ram(to_integer(unsigned(ADDRA) mod 512));

      end if;
    end if;
  end process;


  process(CLKB)
  begin
    if rising_edge(CLKB) then
      if(RWB = '0') then
        ram(to_integer(unsigned(ADDRB) mod 512)) := DINB;
      else
        DOUTB <= ram(to_integer(unsigned(ADDRB) mod 512));
      end if;
    end if;
  end process;

end RAM4K9;
--=================================================================================================
--                                        architecture end
--=================================================================================================
---------------------------------------------------------------------------------------------------
--                                      E N D   O F   F I L E
---------------------------------------------------------------------------------------------------