--_________________________________________________________________________________________________
--                                                                                                |
--                                        |SPEC masterFIP|                                        |
--                                                                                                |
--                                         CERN, BE/CO-HT                                         |
--________________________________________________________________________________________________|

---------------------------------------------------------------------------------------------------
--                                                                                                |
--                                           tb_masterFIP                                         |
--                                                                                                |
---------------------------------------------------------------------------------------------------
-- File         tb_masterFIP.vhd                                                                  |
--                                                                                                |
-- Description  Simple test bench of the masterFIP design. The test bench is triggering the       |
--              fmc_masterfip_core through the PCIe host interface, that is not passing through   |
--              Mock Turtle. The WorldFIP exchange, as specified in the masterFIP_test.vec        |
--              consists of:                                                                      |
--              fmc_masterfip_core: sends ID_DAT                                                  |
--              fmc_masterfip_core: sends RD_DAT                                                  |
--              fmc_masterfip_core: sends ID_DAT                                                  |
--              fmc_masterfip_core: expects RD_DAT from the nanoFIP node                          |
--              etc..                                                                             |
--                                                                                                |
-- Platform      FPGA-generic                                                                     |
--                                                                                                |
---------------------------------------------------------------------------------------------------
--                               GNU LESSER GENERAL PUBLIC LICENSE                                |
--                              ------------------------------------                              |
-- This source file is free software; you can redistribute it and/or modify it under the terms of |
-- the GNU Lesser General Public License as published by the Free Software Foundation; either     |
-- version 2.1 of the License, or (at your option) any later version.                             |
-- This source is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;       |
-- without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.      |
-- See the GNU Lesser General Public License for more details.                                    |
-- You should have received a copy of the GNU Lesser General Public License along with this       |
-- source; if not, please download it from http://www.gnu.org/licenses/lgpl-2.1.html              |
---------------------------------------------------------------------------------------------------

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.numeric_std.all;

library nanofip_lib;
use nanofip_lib.all;

library work;
use work.wishbone_pkg.all;
use work.spec_node_pkg.all;
use work.wrn_mqueue_pkg.all;
use work.wr_node_pkg.all;
use work.masterFIP_pkg.all;
use work.masterfip_wbgen2_pkg.all;
use work.gencores_pkg.all;
library unisim;
use unisim.vcomponents.all;


entity tb_masterFIP is
end tb_masterFIP;

architecture behavioral of tb_masterFIP is

    component spec_masterfip_mt
    generic(
        --g_span             : integer :=32;
        --g_width            : integer :=32;
        g_simul   : boolean :=TRUE
    );
    port(
        -- interface with GNUM circuit
        l_rst_n_i      : in  std_logic;
        -- P2L Direction
        p2l_clkp_i : in  std_logic;                      -- Receiver Source Synchronous Clock+
        p2l_clkn_i : in  std_logic;                      -- Receiver Source Synchronous Clock-
        p2l_data_i  : in  std_logic_vector(15 downto 0);  -- Parallel receive data
        p2l_dframe_i: in  std_logic;                      -- Receive Frame
        p2l_valid_i : in  std_logic;                      -- Receive Data Valid
        p2l_rdy_o   : out std_logic;                      -- Rx Buffer Full Flag
        p_wr_req_i  : in  std_logic_vector(1 downto 0);   -- PCIe Write Request
        p_wr_rdy_o  : out std_logic_vector(1 downto 0);   -- PCIe Write Ready
        rx_error_o  : out std_logic;                      -- Receive Error
        vc_rdy_i    : in  std_logic_vector(1 downto 0);   -- Virtual channel ready
        -- L2P Direction
        l2p_clkp_o : out std_logic;                      -- Transmitter Source Synchronous Clock+
        l2p_clkn_o : out std_logic;                      -- Transmitter Source Synchronous Clock-
        l2p_data_o  : out std_logic_vector(15 downto 0);  -- Parallel transmit data
        l2p_dframe_o: out std_logic;                      -- Transmit Data Frame
        l2p_valid_o : out std_logic;                      -- Transmit Data Valid
        l2p_edb_o   : out std_logic;                      -- Packet termination and discard
        l2p_rdy_i   : in  std_logic;                      -- Tx Buffer Full Flag
        l_wr_rdy_i  : in  std_logic_vector(1 downto 0);   -- Local-to-PCIe Write
        p_rd_d_rdy_i: in  std_logic_vector(1 downto 0);   -- PCIe-to-Local Read Response Data Ready
        tx_error_i  : in  std_logic;                      -- Transmit Error
        --irq_p_o     : out std_logic;                      -- Interrupt request pulse to GN4124 GPIO

        ext_sync_i         : in std_logic;

        dac_cs_n_o         : out std_logic_vector(1 downto 0);
        dac_sclk_o         : out std_logic;
        dac_din_o          : out std_logic;

       speed_b0_i         : in    std_logic;
       speed_b1_i         : in    std_logic;
       fd_rxcdn_i         : in    std_logic;
       fd_rxd_i           : in    std_logic;
       fd_txer_i          : in    std_logic;
       fd_wdgn_i          : in    std_logic;
       fd_rstn_o          : out   std_logic;
       fd_txck_o          : out   std_logic;
       fd_txd_o           : out   std_logic;
       fd_txena_o         : out   std_logic;

       fmc_prsnt_m2c_n_i           : in std_logic;

       clk_125m_pllref_p_i : in   std_logic;
       clk_125m_pllref_n_i : in   std_logic
    );
    end component;


-----------------------------------------------------------------------------
-- GN4124 Local Bus Model
-----------------------------------------------------------------------------
  component GN412X_BFM
    generic
      (
        STRING_MAX     : integer := 256;           -- Command string maximum length
        T_LCLK         : time    := 10 ns;         -- Local Bus Clock Period
        T_P2L_CLK_DLY  : time    := 2 ns;          -- Delay from LCLK to P2L_CLK
        INSTANCE_LABEL : string  := "GN412X_BFM";  -- Label string to be used as a prefix for messages from the model
        MODE_PRIMARY   : boolean := true           -- TRUE for BFM acting as GN412x, FALSE for BFM acting as the DUT
        );
    port
      (
        --=========================================================--
        -------------------------------------------------------------
        -- CMD_ROUTER Interface
        --
        CMD                : in    string(1 to STRING_MAX);
        CMD_REQ            : in    bit;
        CMD_ACK            : out   bit;
        CMD_CLOCK_EN       : in    boolean;
        --=========================================================--
        -------------------------------------------------------------
        -- GN412x Signal I/O
        -------------------------------------------------------------
        -- This is the reset input to the BFM
        --
        RSTINn             : in    std_logic;
        -------------------------------------------------------------
        -- Reset outputs to DUT
        --
        RSTOUT18n          : out   std_logic;
        RSTOUT33n          : out   std_logic;
        -------------------------------------------------------------
        ----------------- Local Bus Clock ---------------------------
        -------------------------------------------------------------  __ Direction for primary mode
        --                                                            / \
        LCLK, LCLKn        : inout std_logic;      -- Out
        -------------------------------------------------------------
        ----------------- Local-to-PCI Dataflow ---------------------
        -------------------------------------------------------------
        -- Transmitter Source Synchronous Clock.
        --
        L2P_CLKp, L2P_CLKn : inout std_logic;      -- In  
        -------------------------------------------------------------
        -- L2P DDR Link
        --
        L2P_DATA           : inout std_logic_vector(15 downto 0);  -- In  -- Parallel Transmit Data.
        L2P_DFRAME         : inout std_logic;  -- In  -- Transmit Data Frame.
        L2P_VALID          : inout std_logic;  -- In  -- Transmit Data Valid. 
        L2P_EDB            : inout std_logic;  -- In  -- End-of-Packet Bad Flag.
        -------------------------------------------------------------
        -- L2P SDR Controls
        --
        L_WR_RDY           : inout std_logic_vector(1 downto 0);  -- Out -- Local-to-PCIe Write.
        P_RD_D_RDY         : inout std_logic_vector(1 downto 0);  -- Out -- PCIe-to-Local Read Response Data Ready.
        L2P_RDY            : inout std_logic;  -- Out -- Tx Buffer Full Flag.
        TX_ERROR           : inout std_logic;  -- Out -- Transmit Error.
        -------------------------------------------------------------
        ----------------- PCIe-to-Local Dataflow ---------------------
        -------------------------------------------------------------
        -- Transmitter Source Synchronous Clock.
        --
        P2L_CLKp, P2L_CLKn : inout std_logic;  -- Out -- P2L Source Synchronous Clock.
        -------------------------------------------------------------
        -- P2L DDR Link
        --
        P2L_DATA           : inout std_logic_vector(15 downto 0);  -- Out -- Parallel Receive Data.
        P2L_DFRAME         : inout std_logic;  -- Out -- Receive Frame.
        P2L_VALID          : inout std_logic;  -- Out -- Receive Data Valid.
        -------------------------------------------------------------
        -- P2L SDR Controls
        --
        P2L_RDY            : inout std_logic;  -- In  -- Rx Buffer Full Flag.
        P_WR_REQ           : inout std_logic_vector(1 downto 0);  -- Out -- PCIe Write Request.
        P_WR_RDY           : inout std_logic_vector(1 downto 0);  -- In  -- PCIe Write Ready.
        RX_ERROR           : inout std_logic;  -- In  -- Receive Error.
        VC_RDY             : inout std_logic_vector(1 downto 0);  -- Out -- Virtual Channel Ready Status.
        -------------------------------------------------------------
        -- GPIO signals
        --
        GPIO               : inout std_logic_vector(15 downto 0)
        );
  end component;  --GN412X_BFM;



component nanofip
port
  (
  c_id_i     : in std_logic_vector (3 downto 0);
  m_id_i     : in std_logic_vector (3 downto 0);
  p3_lgth_i  : in std_logic_vector (2 downto 0);
  rate_i     : in std_logic_vector (1 downto 0);
  subs_i     : in std_logic_vector (7 downto 0);
  fd_rxcdn_i : in std_logic;
  fd_rxd_i   : in std_logic;
  fd_txer_i  : in std_logic;
  fd_wdgn_i  : in std_logic;
  nostat_i   : in std_logic;
  rstin_i    : in std_logic;
  rstpon_i   : in std_logic;
  slone_i    : in std_logic;
  uclk_i     : in std_logic;
  var1_acc_i : in std_logic;
  var2_acc_i : in std_logic;
  var3_acc_i : in std_logic;
  wclk_i     : in std_logic;
  adr_i      : in std_logic_vector (9 downto 0);
  cyc_i      : in std_logic;
  dat_i      : in std_logic_vector (15 downto 0);
  rst_i      : in std_logic;
  stb_i      : in std_logic;
  we_i       : in std_logic;
  jc_tdo_i   : in std_logic;
  s_id_o     : out std_logic_vector (1 downto 0);

  fd_rstn_o  : out std_logic;
  fd_txck_o  : out std_logic;
  fd_txd_o   : out std_logic;
  fd_txena_o : out std_logic;
  rston_o    : out std_logic;

  r_fcser_o  : out std_logic;
  r_tler_o   : out std_logic;
  u_cacer_o  : out std_logic;
  u_pacer_o  : out std_logic;
  var1_rdy_o : out std_logic;
  var2_rdy_o : out std_logic;
  var3_rdy_o : out std_logic;
  dat_o      : out std_logic_vector (15 downto 0);
  ack_o      : out std_logic;
  jc_tms_o   : out std_logic;
  jc_tdi_o   : out std_logic;
  jc_tck_o   : out std_logic);
end component;

	component user_interface
	port(
		urstn_from_nf		: in std_logic;
		var1_rdy_i			: in std_logic;
		var2_rdy_i			: in std_logic;
		var3_rdy_i			: in std_logic;
        p3_lgth_i  : in std_logic_vector (2 downto 0);

		rstpon_o			: out std_logic;
		uclk_o				: out std_logic;
		urstn_to_nf			: out std_logic;
		var1_acc_o			: out std_logic;
		var2_acc_o			: out std_logic;
		var3_acc_o			: out std_logic;

		ack_i				: in std_logic;
		dat_i				: in std_logic_vector(15 downto 0);

		adr_o				: out std_logic_vector(9 downto 0);
		cyc_o				: out std_logic;
		dat_o				: out std_logic_vector(15 downto 0);
		rst_o				: out std_logic;
		stb_o				: out std_logic;
		wclk_o				: out std_logic;
		we_o				: out std_logic
	);
	end component;

    
    	component board_settings
	port(
		s_id_i			: in std_logic_vector(1 downto 0);
		
		c_id_o			: out std_logic_vector(3 downto 0);
		m_id_o			: out std_logic_vector(3 downto 0);
		nostat_o		: out std_logic;
		p3_lgth_o		: out std_logic_vector(2 downto 0);
		rate_o			: out std_logic_vector(1 downto 0);
		slone_o			: out std_logic;
		subs_o			: out std_logic_vector(7 downto 0)
	);
	end component;
-----------------------------------------------------------------------------
-- CMD_ROUTER component
-----------------------------------------------------------------------------
  component cmd_router
    generic(N_BFM      : integer := 8;
            N_FILES    : integer := 3;
            FIFO_DEPTH : integer := 8;
            STRING_MAX : integer := 256
            );
    port(CMD          : out string(1 to STRING_MAX);
         CMD_REQ      : out bit_vector(N_BFM-1 downto 0);
         CMD_ACK      : in  bit_vector(N_BFM-1 downto 0);
         CMD_ERR      : in  bit_vector(N_BFM-1 downto 0);
         CMD_CLOCK_EN : out boolean
         );
  end component;  --cmd_router;

constant pll_clk_period         : time:= 8 ns;
constant g_width                : integer:= 32;
constant g_span                 : integer:= 32;
constant spec_clk_period        : time:= 50 ns;
constant nanoFIP_clk_period     : time:= 25 ns;

  -- Number of Models receiving commands
  constant N_BFM      : integer                      := 1;  -- 0 : GN412X_BFM in Model Mode
  --                                                        -- 1 : GN412X_BFM in DUT mode
  -- Number of files to feed BFMs
  constant N_FILES    : integer                      := 1;
  --
  -- Depth of the command FIFO for each model
  constant FIFO_DEPTH : integer                      := 16;
  --
  -- Maximum width of a command string
  constant STRING_MAX : integer                      := 256;


  signal fd_rxcdn, fd_rxd, fd_txer, fd_wdgn, fd_rstn, fd_txck, fd_txena, fd_txd : std_logic;
  signal consu_data         : std_logic_vector (15 downto 0);
  signal nanoFIP_rxcdn, nanoFIP_txena, nanoFIP_txck, nanoFIP_rstno ,nanoFIP_wdgn : std_logic;
  signal nanoFIP_clk : std_logic;
  signal clk_125m_pllref_p : std_logic := '1';
  signal clk_125m_pllref_n : std_logic := '0';
  
  signal ext_sync : std_logic := '0';

  -- GN4124 interface
  signal rst_n                : std_logic;
  signal RSTINn             : std_logic;
  signal RSTOUT18n          : std_logic;
  signal RSTOUT33n          : std_logic;
  signal LCLK, LCLKn        : std_logic;

  signal P2L_CLKp, P2L_CLKn : std_logic;
  signal P2L_DATA           : std_logic_vector(15 downto 0);
  signal P2L_DATA_32        : std_logic_vector(31 downto 0);  -- For monitoring use
  signal P2L_DFRAME         : std_logic;
  signal P2L_VALID          : std_logic;
  signal P2L_RDY            : std_logic;
  signal P_WR_REQ           : std_logic_vector(1 downto 0);
  signal P_WR_RDY           : std_logic_vector(1 downto 0);
  signal RX_ERROR           : std_logic;
  signal VC_RDY             : std_logic_vector(1 downto 0);
  signal L2P_CLKp, L2P_CLKn : std_logic;
  signal L2P_DATA           : std_logic_vector(15 downto 0);
  signal L2P_DATA_32        : std_logic_vector(31 downto 0);  -- For monitoring use
  signal L2P_DFRAME         : std_logic;
  signal L2P_VALID          : std_logic;
  signal L2P_EDB            : std_logic;
  signal L2P_RDY            : std_logic;
  signal L_WR_RDY           : std_logic_vector(1 downto 0);
  signal P_RD_D_RDY         : std_logic_vector(1 downto 0);
  signal TX_ERROR           : std_logic;
  signal GPIO               : std_logic_vector(15 downto 0);

  	signal rstpon		: std_logic; --! Power On Reset, active low
	signal uclk			: std_logic; --! 40 MHz clock
	signal urst_to_nf	: std_logic; --! Initialisation control, active low
	signal urst_from_nf	: std_logic; --! Reset output, active low
	
	signal var1_rdy		: std_logic; --! Variable 1 ready
	signal var1_acc		: std_logic; --! Variable 1 access
	signal var2_rdy		: std_logic; --! Variable 2 ready
	signal var2_acc		: std_logic; --! Variable 2 access
	signal var3_rdy		: std_logic; --! Variable 3 ready
	signal var3_acc		: std_logic; --! Variable 3 access
	signal u_cacer		: std_logic; --! nanoFIP status byte, bit 2
	signal u_pacer  	: std_logic; --! nanoFIP status byte, bit 3
	signal r_tler   	: std_logic; --! nanoFIP status byte, bit 4
	signal r_fcser  	: std_logic; --! nanoFIP status byte, bit 5

	signal clk			: std_logic:='1';
	signal reset		: std_logic;

	signal ack			: std_logic:='0';
	signal dat_from_fip	: std_logic_vector(15 downto 0);

	signal adr			: std_logic_vector(9 downto 0);
	signal cyc			: std_logic;
	signal dat_to_fip, dat_to_fip_tmp	: std_logic_vector(15 downto 0);

	signal rst			: std_logic;
	signal stb			: std_logic;
	signal wclk			: std_logic;
	signal we			: std_logic;
    	signal rate			: std_logic_vector (1 downto 0); --! Bit rate
	signal subs			: std_logic_vector (7 downto 0); --! Subscriber number coding.
	signal s_id			: std_logic_vector (1 downto 0); --! Identification selection
	signal m_id			: std_logic_vector (3 downto 0); --! Model identification settings
	signal c_id			: std_logic_vector (3 downto 0); --! Constructor identification settings
	signal p3_lgth		: std_logic_vector (2 downto 0); --! Produced variable data length
    signal slone		: std_logic; --! Stand-alone mode
	signal nostat		: std_logic; --! No NanoFIP status transmission

-----------------------------------------------------------------------------
-- Command Router Signals
-----------------------------------------------------------------------------
  signal CMD          : string(1 to STRING_MAX);
  signal CMD_REQ      : bit_vector(N_BFM-1 downto 0);
  signal CMD_ACK      : bit_vector(N_BFM-1 downto 0);
  signal CMD_ERR      : bit_vector(N_BFM-1 downto 0);
  signal CMD_CLOCK_EN : boolean;

begin

    dut: spec_masterfip_mt
    generic map(
        --g_span            => 32,
        --g_width           => 32,
        g_simul  => TRUE
    )
    port map(
        -- interface with GNUM circuit
        l_rst_n_i         => rst_n,
        p2l_clkp_i       => p2l_clkp,
        p2l_clkn_i       => p2l_clkn,
        p2l_data_i        => p2l_data,
        p2l_dframe_i      => p2l_dframe,
        p2l_valid_i       => p2l_valid,
        p2l_rdy_o         => p2l_rdy,
        p_wr_req_i        => p_wr_req,
        p_wr_rdy_o        => p_wr_rdy,
        rx_error_o        => rx_error,
        vc_rdy_i          => vc_rdy,
        l2p_clkp_o       => l2p_clkp,
        l2p_clkn_o       => l2p_clkn,
        l2p_data_o        => l2p_data,
        l2p_dframe_o      => l2p_dframe,
        l2p_valid_o       => l2p_valid,
        l2p_edb_o         => l2p_edb,
        l2p_rdy_i         => l2p_rdy,
        l_wr_rdy_i        => l_wr_rdy,
        p_rd_d_rdy_i      => p_rd_d_rdy,
        tx_error_i        => tx_error,
		  
		ext_sync_i        => ext_sync,
        
        speed_b0_i        => '1',
        speed_b1_i        => '0',

        fd_rxcdn_i        => fd_rxcdn,
        fd_rxd_i          => fd_rxd,
        fd_txer_i         => fd_txer,
        fd_wdgn_i         => fd_wdgn,
        fd_rstn_o         => fd_rstn,
        fd_txck_o         => fd_txck,
        fd_txd_o          => fd_txd,
        fd_txena_o        => fd_txena,

       -- other signals on the spec card
        fmc_prsnt_m2c_n_i     => '0',

     clk_125m_pllref_p_i => clk_125m_pllref_p,
     clk_125m_pllref_n_i => clk_125m_pllref_n);



  agent: nanofip
    port map(
      uclk_i     => nanoFIP_clk,
      
      c_id_i     => "1011",
      m_id_i     => "0101",
      p3_lgth_i  => p3_lgth,
      rate_i     => rate,
      subs_i     => subs,

      fd_rxcdn_i => nanoFIP_rxcdn,
      fd_rxd_i   => fd_txd,
      fd_txer_i  => '0',
      fd_wdgn_i  => nanoFIP_wdgn,

      fd_rstn_o  => nanoFIP_rstno,
      fd_txck_o  => nanoFIP_txck,
      fd_txd_o   => fd_rxd,
      fd_txena_o => nanoFIP_txena,

      nostat_i   => nostat,
      rstin_i    => rst,
      rstpon_i   => '1',
      slone_i    => slone,
      rston_o    => urst_from_nf,
      
      var1_acc_i => var1_acc,
      var2_acc_i => var2_acc,
      var3_acc_i => var3_acc,
      
      wclk_i     => wclk,
      rst_i      => rst,  
      ack_o      => ack,
      adr_i      => adr,
      cyc_i      => cyc,      
      dat_i      => dat_to_fip,
      dat_o      => dat_from_fip,
      stb_i      => stb,  
      we_i       => we,
 
      var1_rdy_o => var1_rdy,
      var2_rdy_o => var2_rdy,
      var3_rdy_o => var3_rdy,
      
      s_id_o     => open,
      r_fcser_o  => open,
      r_tler_o   => open,
      u_cacer_o  => open,
      u_pacer_o  => open,

      jc_tdo_i   => '0',      
      jc_tms_o   => open,
      jc_tdi_o   => open,
      jc_tck_o   => open);

        dat_to_fip <= dat_to_fip_tmp when slone = '0' else dat_from_fip;
      
	user_logic:  user_interface
	port map(
		uclk_o			=> nanoFIP_clk,
        urstn_from_nf	=> urst_from_nf,
		rstpon_o		=> open,
		urstn_to_nf		=> open,

		var1_rdy_i		=> var1_rdy,
		var2_rdy_i		=> var2_rdy,
		var3_rdy_i		=> var3_rdy,        
        p3_lgth_i		=> p3_lgth,
        
		var1_acc_o		=> var1_acc,
		var2_acc_o		=> var2_acc,
		var3_acc_o		=> var3_acc,

		ack_i			=> ack,
		dat_i			=> dat_from_fip,

		adr_o			=> adr,
		cyc_o			=> cyc,
		dat_o			=> dat_to_fip_tmp,
		rst_o			=> open,
		stb_o			=> stb,
		wclk_o			=> wclk,
		we_o			=> we
	);
      
	board: board_settings
	port map(
		s_id_i			=> s_id,

		c_id_o			=> c_id,
		m_id_o			=> m_id,
		nostat_o		=> nostat,
		p3_lgth_o		=> p3_lgth,
		rate_o			=> rate,
		slone_o			=> slone,
		subs_o			=> subs
	);
    
  CMD_ERR <= (others => '0');

  UC : cmd_router
    generic map
    (N_BFM      => N_BFM,
     N_FILES    => N_FILES,
     FIFO_DEPTH => FIFO_DEPTH,
     STRING_MAX => STRING_MAX
     )
    port map
    (CMD          => CMD,
     CMD_REQ      => CMD_REQ,
     CMD_ACK      => CMD_ACK,
     CMD_ERR      => CMD_ERR,
     CMD_CLOCK_EN => CMD_CLOCK_EN
     );

-----------------------------------------------------------------------------
-- GN412x BFM - PRIMARY
-----------------------------------------------------------------------------

  U0 : gn412x_bfm
    generic map
    (
      STRING_MAX     => STRING_MAX,
--      T_LCLK         => 5 ns,
--      T_LCLK         => 10 ns,
      T_LCLK         => 6.25 ns,
      T_P2L_CLK_DLY  => 2 ns,
      INSTANCE_LABEL => "U0(Primary GN412x): ",
      MODE_PRIMARY   => true
      )
    port map
    (
      --=========================================================--
      -------------------------------------------------------------
      -- CMD_ROUTER Interface
      --
      CMD          => CMD,
      CMD_REQ      => CMD_REQ(0),
      CMD_ACK      => CMD_ACK(0),
      CMD_CLOCK_EN => CMD_CLOCK_EN,
      --=========================================================--
      -------------------------------------------------------------
      -- GN412x Signal I/O
      -------------------------------------------------------------
      -- This is the reset input to the BFM
      --
      RSTINn       => RSTINn,
      -------------------------------------------------------------
      -- Reset outputs to DUT
      --
      RSTOUT18n    => RSTOUT18n,
      RSTOUT33n    => RSTOUT33n,
      -------------------------------------------------------------
      ----------------- Local Bus Clock ---------------------------
      ------------------------------------------------------------- 
      --
      LCLK         => LCLK,
      LCLKn        => LCLKn,
      -------------------------------------------------------------
      ----------------- Local-to-PCI Dataflow ---------------------
      -------------------------------------------------------------
      -- Transmitter Source Synchronous Clock.
      --
      L2P_CLKp     => L2P_CLKp,
      L2P_CLKn     => L2P_CLKn,
      -------------------------------------------------------------
      -- L2P DDR Link
      --
      L2P_DATA     => L2P_DATA,
      L2P_DFRAME   => L2P_DFRAME,
      L2P_VALID    => L2P_VALID,
      L2P_EDB      => L2P_EDB,
      -------------------------------------------------------------
      -- L2P SDR Controls
      --
      L_WR_RDY     => L_WR_RDY,
      P_RD_D_RDY   => P_RD_D_RDY,
      L2P_RDY      => L2P_RDY,
      TX_ERROR     => TX_ERROR,
      -------------------------------------------------------------
      ----------------- PCIe-to-Local Dataflow ---------------------
      -------------------------------------------------------------
      -- Transmitter Source Synchronous Clock.
      --
      P2L_CLKp     => P2L_CLKp,
      P2L_CLKn     => P2L_CLKn,
      -------------------------------------------------------------
      -- P2L DDR Link
      --
      P2L_DATA     => P2L_DATA,
      P2L_DFRAME   => P2L_DFRAME,
      P2L_VALID    => P2L_VALID,
      -------------------------------------------------------------
      -- P2L SDR Controls
      --
      P2L_RDY      => P2L_RDY,
      P_WR_REQ     => P_WR_REQ,
      P_WR_RDY     => P_WR_RDY,
      RX_ERROR     => RX_ERROR,
      VC_RDY       => VC_RDY,
      GPIO         => gpio
      );                                -- GN412X_BFM;

    -- tdc_pll_clock: process
    -- begin
        -- if pll_cs_o ='1' and rst_n ='1' then
            -- tdc_clk_p_i         <= not (tdc_clk_p_i) after 1 ns;
            -- tdc_clk_n_i         <= not (tdc_clk_n_i) after 1 ns;

            -- pll_status_i        <= '1';

        -- end if;
        -- wait for pll_clk_period/2;
    -- end process;

    -- tdc_ref_clock: process
    -- begin
        -- if pll_cs_o ='1' and rst_n ='1' then
            -- acam_refclk_i   <= not (acam_refclk_i) after 3 ns;
        -- end if;
        -- wait for pll_clk_period*2;
    -- end process;
    -- acam_refclk_n_i <= not acam_refclk_i;

    
    spec_clock: process
    begin
        clk_125m_pllref_p   <= not (clk_125m_pllref_p) after 1 ns;
        clk_125m_pllref_n   <= not (clk_125m_pllref_n) after 1 ns;
        wait for pll_clk_period/2;
    end process;

     -- nanoFIP_clock: process
     -- begin
         -- nanoFIP_clk           <= not (nanoFIP_clk) after 10 ns;
         -- wait for nanoFIP_clk_period/2;
     -- end process;
    
    ext_sync   <= '1' after 8500   ns, '0' after 8580   ns,
                  '1' after 194000 ns, '0' after 194080 ns,
                  '1' after 565151 ns, '0' after 565231 ns;

    fd_wdgn   <= '0' after 8500   ns, '1' after 8580   ns,
                  '0' after 194000 ns, '1' after 194080 ns,
                  '0' after 565151 ns, '1' after 565231 ns;

    rst_n               <= RSTOUT18n;
    rst                 <= not rst_n;
    
end behavioral;
